@@ -1,7 +1,7 @@
 /*
  *      mmguicore.h
  *      
- *      Copyright 2013 Alex <alex@linuxonly.ru>
+ *      Copyright 2013-2014 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -27,13 +27,15 @@
 #include "netlink.h"
 #include "smsdb.h"
 
-#define MMGUI_SPEED_VALUES_NUMBER  20
+#define MMGUI_SPEED_VALUES_NUMBER    20
 
-#define MMGUI_THREAD_SLEEP_PERIOD   1 /*seconds*/
+#define MMGUI_THREAD_SLEEP_PERIOD    1 /*seconds*/
 
 enum _mmgui_event {
 	MMGUI_EVENT_DEVICE_ADDED = 0,
 	MMGUI_EVENT_DEVICE_REMOVED,
+	MMGUI_EVENT_DEVICE_OPENED,
+	MMGUI_EVENT_DEVICE_CLOSING,
 	MMGUI_EVENT_DEVICE_ENABLED_STATUS,
 	MMGUI_EVENT_DEVICE_BLOCKED_STATUS,
 	MMGUI_EVENT_DEVICE_CONNECTION_STATUS,
@@ -42,6 +44,8 @@ enum _mmgui_event {
 	MMGUI_EVENT_SMS_COMPLETED,
 	MMGUI_EVENT_SMS_SENT,
 	MMGUI_EVENT_SMS_REMOVED,
+	MMGUI_EVENT_SMS_NEW_DAY,
+	
 	MMGUI_EVENT_SIGNAL_LEVEL_CHANGE,
 	MMGUI_EVENT_NETWORK_MODE_CHANGE,
 	MMGUI_EVENT_NETWORK_REGISTRATION_CHANGE,
@@ -54,6 +58,8 @@ enum _mmgui_event {
 	MMGUI_EVENT_TRAFFIC_LIMIT,
 	MMGUI_EVENT_TIME_LIMIT,
 	MMGUI_EVENT_UPDATE_CONNECTIONS_LIST,
+	
+	MMGUI_EVENT_EXTEND_CAPABILITIES
 		
 };
 
@@ -64,9 +70,9 @@ typedef void (*mmgui_event_int_callback)(enum _mmgui_event event, gpointer mmgui
 
 enum _mmgui_module_priority {
 	//module priority levels
-	MMGUI_MODULE_PRIORITY_LOW = 0x00,
-	MMGUI_MODULE_PRIORITY_NORMAL,
-	MMGUI_MODULE_PRIORITY_RECOMMENDED
+	MMGUI_MODULE_PRIORITY_LOW         = 0,
+	MMGUI_MODULE_PRIORITY_NORMAL      = 1,
+	MMGUI_MODULE_PRIORITY_RECOMMENDED = 2
 };
 
 enum _mmgui_module_type {
@@ -91,6 +97,9 @@ enum _mmgui_module_requirement {
 struct _mmguimodule {
 	//module description
 	guint identifier;
+	gboolean applicable;
+	gboolean activatable;
+	gboolean recommended;
 	enum _mmgui_module_type type;
 	enum _mmgui_module_requirement requirement;
 	enum _mmgui_module_priority priority;
@@ -98,6 +107,7 @@ struct _mmguimodule {
 	gchar servicename[256];
 	gchar description[256];
 	gchar filename[256];
+	gchar shortname[256];
 };
 
 typedef struct _mmguimodule *mmguimodule_t;
@@ -197,6 +207,15 @@ enum _mmgui_contacts_storage {
 };
 
 /*Capbilities specifications*/
+enum _mmgui_capabilities {
+	MMGUI_CAPS_NONE                  = 0,
+	MMGUI_CAPS_SMS                   = 1 << 1,
+	MMGUI_CAPS_USSD                  = 1 << 2,
+	MMGUI_CAPS_LOCATION              = 1 << 3,
+	MMGUI_CAPS_SCAN                  = 1 << 4,
+	MMGUI_CAPS_CONTACTS              = 1 << 5,
+};
+
 enum _mmgui_sms_capabilities {
 	MMGUI_SMS_CAPS_NONE              = 0,
     MMGUI_SMS_CAPS_RECEIVE           = 1 << 1,
@@ -257,23 +276,6 @@ struct _mmgui_scanned_network {
 
 typedef struct _mmgui_scanned_network *mmgui_scanned_network_t;
 
-struct _mmgui_ext_connection {
-	guchar state;
-	guchar family;
-	gushort srcport;
-	guint dqueue;
-	guint inode;
-	guint userid;
-	guint apppid;
-	gchar *appname;
-	gchar *dsthostname;
-	gchar srcaddr[INET6_ADDRSTRLEN + 8];
-	gchar dstaddr[INET6_ADDRSTRLEN + 8];
-	gboolean existsflag;
-};
-
-typedef struct _mmgui_ext_connection *mmgui_ext_connection_t;
-
 struct _mmgui_contact {
 	guint id;        //Internal private number
 	gchar *name;     //Full name of the contact
@@ -288,25 +290,6 @@ struct _mmgui_contact {
 
 typedef struct _mmgui_contact *mmgui_contact_t;
 
-struct _mmgui_traffic_limits {
-	gboolean trafficenabled;
-	gboolean trafficexecuted;
-	guint trafficamount;
-	guint trafficunits;
-	guint64 trafficfull;
-	gchar *trafficmessage;
-	guint trafficaction;
-	gboolean timeenabled;
-	gboolean timeexecuted;
-	guint timeamount;
-	guint timeunits;
-	guint64 timefull;
-	gchar *timemessage;
-	guint timeaction;
-};
-
-typedef struct _mmgui_traffic_limits *mmgui_traffic_limits_t;
-
 struct _mmgui_traffic_stats {
 	gint64 rxbytes;
 	gint64 rxpackets;
@@ -328,12 +311,41 @@ struct _mmgui_traffic_stats {
 
 typedef struct _mmgui_traffic_stats *mmgui_traffic_stats_t;
 
+struct _mmgui_core_options {
+	/*Preferred modules*/
+	gchar *mmmodule;
+	gchar *cmmodule;
+	/*Timeouts*/
+	gint enabletimeout;
+	gint sendsmstimeout;
+	gint sendussdtimeout;
+	gint scannetworkstimeout;
+	/*Traffic limits*/
+	gboolean trafficenabled;
+	gboolean trafficexecuted;
+	guint trafficamount;
+	guint trafficunits;
+	guint64 trafficfull;
+	gchar *trafficmessage;
+	guint trafficaction;
+	gboolean timeenabled;
+	gboolean timeexecuted;
+	guint timeamount;
+	guint timeunits;
+	guint64 timefull;
+	gchar *timemessage;
+	guint timeaction;
+};
+
+typedef struct _mmgui_core_options *mmgui_core_options_t;
+
 struct _mmguidevice {
 	guint id;
 	/*State*/
 	gboolean enabled;
 	gboolean blocked;
 	gboolean registered;
+	//gboolean powered;
 	enum _mmgui_device_operation operation;
 	//Info
 	gchar *manufacturer;
@@ -347,7 +359,8 @@ struct _mmguidevice {
 	enum _mmgui_device_types type;
 	gchar *imei;
 	gchar *imsi;
-	gchar *operatorcode;
+	//gchar *operatorcode;
+	gint operatorcode; /*mcc/mnc*/
 	gchar *operatorname;
 	enum _mmgui_reg_status regstatus;
 	guint allmode;
@@ -401,10 +414,10 @@ typedef gboolean (*mmgui_module_devices_enable_func)(gpointer mmguicore, gboolea
 typedef guint (*mmgui_module_sms_enum_func)(gpointer mmguicore, GSList **smslist);
 typedef mmgui_sms_message_t (*mmgui_module_sms_get_func)(gpointer mmguicore, guint index);
 typedef gboolean (*mmgui_module_sms_delete_func)(gpointer mmguicore, guint index);
-typedef gboolean (*mmgui_module_sms_send_func)(gpointer mmguicore, gchar* number, gchar *text);
+typedef gboolean (*mmgui_module_sms_send_func)(gpointer mmguicore, gchar* number, gchar *text, gint validity, gboolean report);
 typedef gboolean (*mmgui_module_ussd_cancel_session_func)(gpointer mmguicore);
 typedef enum _mmgui_ussd_state (*mmgui_module_ussd_get_state_func)(gpointer mmguicore);
-typedef gboolean (*mmgui_module_ussd_send_func)(gpointer mmguicore, gchar *request, enum _mmgui_ussd_validation validationid);
+typedef gboolean (*mmgui_module_ussd_send_func)(gpointer mmguicore, gchar *request, enum _mmgui_ussd_validation validationid, gboolean reencode);
 typedef gboolean (*mmgui_module_networks_scan_func)(gpointer mmguicore);
 typedef guint (*mmgui_module_contacts_enum_func)(gpointer mmguicore, GSList **contactslist);
 typedef gboolean (*mmgui_module_contacts_delete_func)(gpointer mmguicore, guint index);
@@ -422,6 +435,11 @@ typedef gboolean (*mmgui_module_device_connection_disconnect_func)(gpointer mmgu
 struct _mmguicore {
 	//Modules list
 	GSList *modules;
+	//Cache
+	gchar *cachefilename;
+	GKeyFile *cachekeyfile;
+	gboolean updatecache;
+	time_t updcachetime;
 	//Modem manager module
 	GModule *module;
 	gpointer moduledata;
@@ -469,13 +487,14 @@ struct _mmguicore {
 	//Callbacks
 	mmgui_event_int_callback eventcb;
 	mmgui_event_ext_callback extcb;
-	//Traffic limits
-	mmgui_traffic_limits_t limits;
+	/*Core options*/
+	mmgui_core_options_t options;
 	//User data pointer
 	gpointer userdata;
 	//Netlink interface
 	mmgui_netlink_t netlink;
-	GHashTable *connections;
+	/*New day time*/
+	time_t newdaytime;
 	//Work thread
 	GThread *workthread;
 	gint workthreadctl[2];
@@ -490,9 +509,13 @@ struct _mmguicore {
 
 typedef struct _mmguicore *mmguicore_t;
 
+/*Modules*/
+//void mmguicore_modules_list(mmguicore_t mmguicore, mmgui_modules_enum_callback callback);
+GSList *mmguicore_modules_info(void);
+void mmguicore_modules_mm_set_timeouts(mmguicore_t mmguicore, gint operation1, gint timeout1, ...);
 //Devices
+gboolean mmguicore_devices_enum(mmguicore_t mmguicore);
 gboolean mmguicore_devices_open(mmguicore_t mmguicore, guint deviceid, gboolean openfirst);
-
 gboolean mmguicore_devices_enable(mmguicore_t mmguicore, gboolean enabled);
 GSList *mmguicore_devices_get_list(mmguicore_t mmguicore);
 mmguidevice_t mmguicore_devices_get_current(mmguicore_t mmguicore);
@@ -517,7 +540,7 @@ GSList *mmguicore_sms_enum(mmguicore_t mmguicore, gboolean concatenation);
 mmgui_sms_message_t mmguicore_sms_get(mmguicore_t mmguicore, guint index);
 gboolean mmguicore_sms_delete(mmguicore_t mmguicore, guint index);
 gboolean mmguicore_sms_validate_number(const gchar *number);
-gboolean mmguicore_sms_send(mmguicore_t mmguicore, gchar *number, gchar *text);
+gboolean mmguicore_sms_send(mmguicore_t mmguicore, gchar *number, gchar *text, gint validity, gboolean report);
 //USSD
 guint mmguicore_ussd_get_capabilities(mmguicore_t mmguicore);
 enum _mmgui_ussd_validation mmguicore_ussd_validate_request(gchar *request);
@@ -538,12 +561,14 @@ mmgui_contact_t mmguicore_contacts_get(mmguicore_t mmguicore, guint index);
 gboolean mmguicore_contacts_delete(mmguicore_t mmguicore, guint index);
 gboolean mmguicore_contacts_add(mmguicore_t mmguicore, mmgui_contact_t contact);
 //Connections
-GHashTable *mmguicore_connections_sync(mmguicore_t mmguicore);
+GSList *mmguicore_open_connections_list(mmguicore_t mmguicore);
+void mmguicore_close_connections_list(mmguicore_t mmguicore);
+GSList *mmguicore_get_connections_changes(mmguicore_t mmguicore);
 //MMGUI Core
 gchar *mmguicore_get_last_error(mmguicore_t mmguicore);
 gchar *mmguicore_get_last_connection_error(mmguicore_t mmguicore);
 gboolean mmguicore_interrupt_operation(mmguicore_t mmguicore);
-mmguicore_t mmguicore_new(mmgui_event_ext_callback callback, mmgui_traffic_limits_t limits, gpointer userdata);
+mmguicore_t mmguicore_new(mmgui_event_ext_callback callback/*, mmgui_connections_sync_callback connsynccb*/, mmgui_core_options_t options, gpointer userdata);
 void mmguicore_close(mmguicore_t mmguicore);
 
 #endif /* __MMGUICORE_H__ */
