@@ -1,7 +1,7 @@
 /*
  *      mm06.c
  *      
- *      Copyright 2013 Alex <alex@linuxonly.ru>
+ *      Copyright 2013-2014 Alex <alex@linuxonly.ru>
  *      
  *      This program is free software: you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,14 +19,17 @@
 
 
 #include <glib.h>
+#include <glib/gprintf.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gmodule.h>
 #include <string.h>
+#include <stdlib.h>
 #include <net/if.h>
 
 #include "../mmguicore.h"
 #include "../smsdb.h"
+#include "../encoding.h"
 
 #define MMGUI_MODULE_SERVICE_NAME  "org.freedesktop.ModemManager"
 #define MMGUI_MODULE_IDENTIFIER    60
@@ -91,17 +94,18 @@ struct _mmguimoduledata {
 	GDBusProxy *timeproxy;
 	GDBusProxy *contactsproxy;
 	//Attached signal handlers
-	gulong netsignal;
 	gulong statesignal;
 	gulong smssignal;
 	//Property change signal
-	gulong modempropsignal;
+	gulong netpropsignal;
 	gulong locationpropsignal;
 	//Service type
 	ModuleIntService service;
 	//Legacy ModemManager versions
 	gboolean needsmspolling;
 	time_t polltimestamp;
+	//USSD reencoding flag
+	gboolean reencodeussd;
 	//Last error message
 	gchar *errormessage;
 	//Cancellable
@@ -112,12 +116,14 @@ struct _mmguimoduledata {
 
 typedef struct _mmguimoduledata *moduledata_t;
 
+static void mmgui_module_handle_error_message(mmguicore_t mmguicore, GError *error);
+static guint mmgui_module_device_id(const gchar *devpath);
+static gint mmgui_module_gsm_operator_code(const gchar *opcodestr);
 
 static gboolean mmgui_module_device_registered_from_status(guint status);
 static mmguidevice_t mmgui_module_device_new(mmguicore_t mmguicore, const gchar *devpath);
 static gboolean mmgui_module_devices_update_registration(gpointer mmguicore, guint regstatus, gchar *operatorcode, gchar *operatorname);
 static gboolean mmgui_module_devices_update_device_mode(gpointer mmguicore, gint oldstate, gint newstate, guint changereason);
-static gboolean mmgui_module_devices_update_mode(gpointer mmguicore);
 static gboolean mmgui_module_devices_update_location(gpointer mmguicore, mmguidevice_t device);
 static gboolean mmgui_module_devices_enable_location(gpointer mmguicore, mmguidevice_t device, gboolean enable);
 
@@ -159,6 +165,33 @@ static guint mmgui_module_device_id(const gchar *devpath)
 	return id;
 }
 
+static gint mmgui_module_gsm_operator_code(const gchar *opcodestr)
+{
+	gsize length;
+	gchar codepartbuf[4];
+	gint operatorcode;
+	
+	if (opcodestr == NULL) return 0;
+	
+	length = strlen(opcodestr);
+	
+	if (length < 5) return 0;
+	
+	operatorcode = 0;
+	
+	/*MCC*/
+	memset(codepartbuf, 0, sizeof(codepartbuf));
+	memcpy(codepartbuf, opcodestr, 3);
+	operatorcode |= (atoi(codepartbuf) & 0x0000ffff) << 16;
+	
+	/*MNC*/
+	memset(codepartbuf, 0, sizeof(codepartbuf));
+	memcpy(codepartbuf, opcodestr + 3, length - 3);
+	operatorcode |= atoi(codepartbuf) & 0x0000ffff;
+	
+	return operatorcode;
+}
+
 static void mmgui_signal_handler(GDBusProxy *proxy, const gchar *sender_name, const gchar *signal_name, GVariant *parameters, gpointer data)
 {
 	mmguicore_t mmguicore;
@@ -178,7 +211,6 @@ static void mmgui_signal_handler(GDBusProxy *proxy, const gchar *sender_name, co
 		if (g_str_equal(signal_name, "DeviceAdded")) {
 			g_variant_get(parameters, "(o)", &devpath);
 			if (devpath != NULL) {
-				g_printf("Added: %s\n", devpath);
 				device = mmgui_module_device_new(mmguicore, devpath);
 				(mmguicore->eventcb)(MMGUI_EVENT_DEVICE_ADDED, mmguicore, device);
 			}
@@ -199,10 +231,6 @@ static void mmgui_signal_handler(GDBusProxy *proxy, const gchar *sender_name, co
 				mmguicore->device->siglevel = id;
 				(mmguicore->eventcb)(MMGUI_EVENT_SIGNAL_LEVEL_CHANGE, mmguicore, mmguicore->device);
 			}
-		} else if (g_str_equal(signal_name, "NetworkMode")) {
-			if (mmgui_module_devices_update_mode(mmguicore)) {
-				(mmguicore->eventcb)(MMGUI_EVENT_NETWORK_MODE_CHANGE, mmguicore, mmguicore->device);
-			}
 		} else if (g_str_equal(signal_name, "RegistrationInfo")) {
 			if (mmguicore->device != NULL) {
 				g_variant_get(parameters, "(uss)", &regstatus, &operatorcode, &operatorname);
@@ -226,11 +254,8 @@ static void mmgui_property_change_handler(GDBusProxy *proxy, GVariant *changed_p
 	mmguicore_t mmguicore;
 	mmguidevice_t device;
 	GVariantIter *iter;
-	GVariant *item;
 	const gchar *key;
 	GVariant *value;
-	guint statevalue;
-	gboolean stateflag;
 	
 	if ((changed_properties == NULL) || (data == NULL)) return;
 	
@@ -243,11 +268,26 @@ static void mmgui_property_change_handler(GDBusProxy *proxy, GVariant *changed_p
 		g_variant_get(changed_properties, "a{sv}", &iter);
 		while (g_variant_iter_loop(iter, "{&sv}", &key, &value)) {
 			if (g_str_equal(key, "Location")) {
-				if (mmgui_module_devices_update_location(mmguicore, mmguicore->device)) {
-					if (mmguicore->eventcb != NULL) {
-						(mmguicore->eventcb)(MMGUI_EVENT_LOCATION_CHANGE, mmguicore, mmguicore->device);
+				/*Enable location interface and update location*/
+				if (mmgui_module_devices_enable_location(mmguicore, device, TRUE)) {
+					if (mmgui_module_devices_update_location(mmguicore, device)) {
+						if (mmguicore->eventcb != NULL) {
+							(mmguicore->eventcb)(MMGUI_EVENT_LOCATION_CHANGE, mmguicore, device);
+						}
 					}
 				}
+			} else if (g_str_equal(key, "AllowedMode")) {
+				/*Update allowed mode*/
+				device->allmode = g_variant_get_uint32(value);
+				if (mmguicore->eventcb != NULL) {
+					(mmguicore->eventcb)(MMGUI_EVENT_NETWORK_MODE_CHANGE, mmguicore, device);
+				}
+			} else if (g_str_equal(key, "AccessTechnology")) {
+				/*Update access technology*/
+				device->mode = g_variant_get_uint32(value);
+				if (mmguicore->eventcb != NULL) {
+					(mmguicore->eventcb)(MMGUI_EVENT_NETWORK_MODE_CHANGE, mmguicore, device);
+				}
 			}
 			g_debug("Property changed: %s\n", key);
 		}
@@ -393,7 +433,7 @@ static mmguidevice_t mmgui_module_device_new(mmguicore_t mmguicore, const gchar
 	//Networks
 	//Info
 	device->operatorname = NULL;
-	device->operatorcode = NULL;
+	device->operatorcode = 0;
 	device->imei = NULL;
 	device->imsi = NULL;
 	//USSD
@@ -1010,16 +1050,12 @@ static gboolean mmgui_module_devices_update_registration(gpointer mmguicore, gui
 	if (mmguicorelc->device == NULL) return FALSE;
 	device = mmguicorelc->device;
 	
-	if (device->operatorcode != NULL) {
-		g_free(device->operatorcode);
-	}
-	
 	if (device->operatorname != NULL) {
 		g_free(device->operatorname);
 	}
 	
 	device->registered = mmgui_module_device_registered_from_status(regstatus);
-	device->operatorcode = g_strdup(operatorcode);
+	device->operatorcode = mmgui_module_gsm_operator_code(operatorcode);
 	device->operatorname = g_strdup(operatorname);
 	
 	return TRUE;
@@ -1186,39 +1222,6 @@ static gboolean mmgui_module_devices_update_device_mode(gpointer mmguicore, gint
 	return TRUE;
 }
 
-static gboolean mmgui_module_devices_update_mode(gpointer mmguicore)
-{
-	mmguicore_t mmguicorelc;
-	moduledata_t moduledata;
-	mmguidevice_t device;
-	GVariant *data;
-		
-	if (mmguicore == NULL) return FALSE;
-	mmguicorelc = (mmguicore_t)mmguicore;
-	
-	if (mmguicorelc->moduledata == NULL) return FALSE;
-	moduledata = (moduledata_t)mmguicorelc->moduledata;
-	
-	if (mmguicorelc->device == NULL) return FALSE;
-	device = mmguicorelc->device;
-	
-	if (moduledata->netproxy != NULL) {
-		//Allowed mode
-		data = g_dbus_proxy_get_cached_property(moduledata->netproxy, "AllowedMode");
-		device->allmode = g_variant_get_uint32(data);
-		g_variant_unref(data);
-		
-		//Access technology
-		data = g_dbus_proxy_get_cached_property(moduledata->netproxy, "AccessTechnology");
-		device->mode = g_variant_get_uint32(data);
-		g_variant_unref(data);
-		
-		return TRUE;
-	}
-	
-	return FALSE;
-}
-
 static gboolean mmgui_module_devices_update_location(gpointer mmguicore, mmguidevice_t device)
 {
 	mmguicore_t mmguicorelc;
@@ -1348,7 +1351,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 	mmguidevice_t device;
 	GVariant *data;
 	GError *error;
-	gchar *blockstr;
+	gchar *blockstr, *operatorcode;
 	gsize strsize = 256;
 	
 	if (mmguicore == NULL) return FALSE;
@@ -1368,7 +1371,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 			g_variant_unref(data);
 		} else {
 			device->enabled = TRUE;
-			g_printf("Failed to get device enabled state\n");
+			g_debug("Failed to get device enabled state\n");
 		}
 		//Is device blocked
 		data = g_dbus_proxy_get_cached_property(moduledata->modemproxy, "UnlockRequired");
@@ -1378,7 +1381,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 			g_variant_unref(data);
 		} else {
 			device->blocked = FALSE;
-			g_printf("Failed to get device blocked state\n");
+			g_debug("Failed to get device blocked state\n");
 		}
 	}
 	
@@ -1407,10 +1410,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 		}
 		
 		//Operator
-		if (device->operatorcode != NULL) {
-			g_free(device->operatorcode);
-			device->operatorcode = NULL;
-		}
+		device->operatorcode = 0;
 		
 		if (device->operatorname != NULL) {
 			g_free(device->operatorname);
@@ -1433,9 +1433,9 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 			mmgui_module_handle_error_message(mmguicorelc, error);
 			g_error_free(error);
 		} else {
-			g_variant_get(data, "((uss))", &device->regstatus, &device->operatorcode, &device->operatorname);
+			g_variant_get(data, "((uss))", &device->regstatus, &operatorcode, &device->operatorname);
 			device->registered = mmgui_module_device_registered_from_status(device->regstatus);
-			device->operatorcode = g_strdup(device->operatorcode);
+			device->operatorcode = mmgui_module_gsm_operator_code(operatorcode);
 			device->operatorname = g_strdup(device->operatorname);
 			g_variant_unref(data);
 		}
@@ -1447,7 +1447,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 			g_variant_unref(data);
 		} else {
 			device->allmode = 0;
-			g_printf("Failed to get device allowed mode\n");
+			g_debug("Failed to get device allowed mode\n");
 		}
 		
 		//Access technology
@@ -1457,7 +1457,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_information(gpointer mmguicore)
 			g_variant_unref(data);
 		} else {
 			device->mode = 0;
-			g_printf("Failed to get device access mode\n");
+			g_debug("Failed to get device access mode\n");
 		}
 	}
 	
@@ -1648,7 +1648,7 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_open(gpointer mmguicore, mmguidevi
 		g_error_free(error);
 	} else {
 		device->scancaps = MMGUI_SCAN_CAPS_OBSERVE;
-		moduledata->netsignal = g_signal_connect(moduledata->netproxy, "g-signal", G_CALLBACK(mmgui_signal_handler), mmguicore);
+		moduledata->netpropsignal = g_signal_connect(moduledata->netproxy, "g-properties-changed", G_CALLBACK(mmgui_property_change_handler), mmguicore);
 	}
 	//Modem interface
 	error = NULL;
@@ -1724,9 +1724,8 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_open(gpointer mmguicore, mmguidevi
 			mmgui_module_handle_error_message(mmguicorelc, error);
 			g_error_free(error);
 		} else {
-			if (mmgui_module_devices_enable_location(mmguicore, device, TRUE)) {
-				moduledata->locationpropsignal = g_signal_connect(moduledata->locationproxy, "g-properties-changed", G_CALLBACK(mmgui_property_change_handler), mmguicore);
-			}
+			moduledata->locationpropsignal = g_signal_connect(moduledata->locationproxy, "g-properties-changed", G_CALLBACK(mmgui_property_change_handler), mmguicore);
+			mmgui_module_devices_enable_location(mmguicore, device, TRUE);
 		}
 		//Time interface
 		error = NULL;
@@ -1812,8 +1811,8 @@ G_MODULE_EXPORT gboolean mmgui_module_devices_close(gpointer mmguicore)
 		moduledata->cardproxy = NULL;
 	}
 	if (moduledata->netproxy != NULL) {
-		if (g_signal_handler_is_connected(moduledata->netproxy, moduledata->netsignal)) {
-			g_signal_handler_disconnect(moduledata->netproxy, moduledata->netsignal);
+		if (g_signal_handler_is_connected(moduledata->netproxy, moduledata->netpropsignal)) {
+			g_signal_handler_disconnect(moduledata->netproxy, moduledata->netpropsignal);
 		}
 		g_object_unref(moduledata->netproxy);
 		moduledata->netproxy = NULL;
@@ -1905,7 +1904,7 @@ static gboolean mmgui_module_devices_restart_ussd(gpointer mmguicore)
 		}
 	}	
 	
-	
+	return FALSE;
 }
 
 static void mmgui_module_devices_enable_handler(GDBusProxy *proxy, GAsyncResult *res, gpointer user_data)
@@ -1941,6 +1940,12 @@ static void mmgui_module_devices_enable_handler(GDBusProxy *proxy, GAsyncResult
 		g_variant_unref(result);
 		//If device state changed - return TRUE
 		if (newstate != mmguicorelc->device->enabled) {
+			if (mmguicorelc->device->enabled) {
+				/*Enable location interface*/
+				mmgui_module_devices_enable_location(mmguicorelc, mmguicorelc->device, TRUE);
+				/*Update device mode*/
+				//mmgui_module_devices_update_mode(mmguicorelc);
+			}
 			newstate = TRUE;
 		} else {
 			newstate = FALSE;
@@ -2035,8 +2040,9 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 	moduledata_t moduledata;
 	mmgui_sms_message_t message;
 	GVariant *value;
-	gsize strlength;
+	gsize strlength, declength;
 	const gchar *valuestr;
+	gchar *decstr;
 	gboolean gottext, gotindex;
 	guint index;
 	
@@ -2046,14 +2052,27 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 	
 	moduledata = (moduledata_t)mmguicore->moduledata;
 	
+	index = 0;
+	
 	message = mmgui_smsdb_message_create();
 	//Sender number
 	value  = g_variant_lookup_value(messagev, "number", G_VARIANT_TYPE_STRING);
 	if (value != NULL) {
 		strlength = 256;
 		valuestr = g_variant_get_string(value, &strlength);
-		mmgui_smsdb_message_set_number(message, valuestr);
-		g_debug("SMS number: %s\n", valuestr);
+		if (moduledata->needsmspolling) {
+			/*Old ModemManager versions tend to not bcd-decode sender numbers, doing it*/
+			decstr = (gchar *)bcd_to_utf8_ascii_part((const guchar *)valuestr, strlength, &declength);
+			if (decstr != NULL) {
+				mmgui_smsdb_message_set_number(message, decstr);
+				g_debug("SMS number: %s\n", decstr);
+				g_free(decstr);
+			} else {
+				mmgui_smsdb_message_set_number(message, valuestr);
+				g_debug("SMS number: %s\n", valuestr);
+			}
+			
+		}
 		g_variant_unref(value);
 	} else {
 		mmgui_smsdb_message_set_number(message, "--");
@@ -2128,17 +2147,17 @@ static mmgui_sms_message_t mmgui_module_sms_retrieve(mmguicore_t mmguicore, GVar
 			gotindex = TRUE;
 		}
 	}
-	//Return message
+	
 	if ((!gotindex) || (!gottext)) {
 		//Message identifier unknown or no text - skip it
 		mmgui_smsdb_message_free(message);
 		return NULL;
-	} else {
-		//Return message
-		mmgui_smsdb_message_set_identifier(message, index, FALSE);
-		g_debug("SMS index: %u\n", index);
-		return message;
 	}
+	
+	//Return message
+	mmgui_smsdb_message_set_identifier(message, index, FALSE);
+	g_debug("SMS index: %u\n", index);
+	return message;
 }
 
 G_MODULE_EXPORT guint mmgui_module_sms_enum(gpointer mmguicore, GSList **smslist)
@@ -2326,7 +2345,7 @@ static void mmgui_module_sms_send_handler(GDBusProxy *proxy, GAsyncResult *res,
 	}
 }
 
-G_MODULE_EXPORT gboolean mmgui_module_sms_send(gpointer mmguicore, gchar* number, gchar *text)
+G_MODULE_EXPORT gboolean mmgui_module_sms_send(gpointer mmguicore, gchar* number, gchar *text, gint validity, gboolean report)
 {
 	mmguicore_t mmguicorelc;
 	moduledata_t moduledata;
@@ -2349,6 +2368,9 @@ G_MODULE_EXPORT gboolean mmgui_module_sms_send(gpointer mmguicore, gchar* number
 	builder = g_variant_builder_new(G_VARIANT_TYPE_ARRAY);
 	g_variant_builder_add_parsed(builder, "{'number', <%s>}", number);
 	g_variant_builder_add_parsed(builder, "{'text', <%s>}", text);
+	if ((validity > -1) && (validity <= 255)) {
+		g_variant_builder_add_parsed(builder, "{'validity', <%u>}", (guint)validity);
+	}
 	array = g_variant_builder_end(builder);
 	
 	builder = g_variant_builder_new(G_VARIANT_TYPE_TUPLE);
@@ -2482,7 +2504,13 @@ static void mmgui_module_ussd_send_handler(GDBusProxy *proxy, GAsyncResult *res,
 		g_error_free(error);
 	} else {
 		g_variant_get(result, "(s)", &answer);
-		answer = g_strdup(answer);
+		if (moduledata->reencodeussd) {
+			/*Fix answer broken encoding*/
+			answer = encoding_ussd_gsm7_to_ucs2(answer);
+		} else {
+			/*Do not touch answer*/
+			answer = g_strdup(answer);
+		}
 		g_variant_unref(result);
 	}
 		
@@ -2495,7 +2523,7 @@ static void mmgui_module_ussd_send_handler(GDBusProxy *proxy, GAsyncResult *res,
 	}
 }
 
-G_MODULE_EXPORT gboolean mmgui_module_ussd_send(gpointer mmguicore, gchar *request, enum _mmgui_ussd_validation validationid)
+G_MODULE_EXPORT gboolean mmgui_module_ussd_send(gpointer mmguicore, gchar *request, enum _mmgui_ussd_validation validationid, gboolean reencode)
 {
 	mmguicore_t mmguicorelc;
 	moduledata_t moduledata;
@@ -2523,6 +2551,8 @@ G_MODULE_EXPORT gboolean mmgui_module_ussd_send(gpointer mmguicore, gchar *reque
 	
 	ussdreq = g_variant_new("(s)", request);
 	
+	command = NULL;
+	
 	if (sessionstate == MMGUI_USSD_STATE_IDLE) {
 		command = "Initiate";
 	} else if (sessionstate == MMGUI_USSD_STATE_USER_RESPONSE) {
@@ -2534,6 +2564,8 @@ G_MODULE_EXPORT gboolean mmgui_module_ussd_send(gpointer mmguicore, gchar *reque
 		}
 	}
 	
+	moduledata->reencodeussd = reencode;
+	
 	mmguicorelc->device->operation = MMGUI_DEVICE_OPERATION_SEND_USSD;
 	
 	if (moduledata->cancellable != NULL) {
@@ -2558,7 +2590,7 @@ static mmgui_scanned_network_t mmgui_module_network_retrieve(GVariant *networkv)
 	GVariant *value;
 	gsize strlength;
 	const gchar *valuestr;
-	guint i;
+	/*guint i;*/
 	
 	if (networkv == NULL) return NULL;
 		
