@@ -190,8 +190,7 @@ G_MODULE_EXPORT gboolean mmgui_module_connection_close(gpointer mmguicore)
 {
 	mmguicore_t mmguicorelc;
 	moduledata_t moduledata;
-	GError *error;
-	
+		
 	if (mmguicore == NULL) return FALSE;
 	mmguicorelc = (mmguicore_t)mmguicore;
 	moduledata = (moduledata_t)(mmguicorelc->cmoduledata);
@@ -242,7 +241,7 @@ G_MODULE_EXPORT gboolean mmgui_module_device_connection_open(gpointer mmguicore,
 	const gchar *valuestr;
 	GDBusProxy *nmdevproxy;
 	GVariant *devproperties;
-	const gchar *nmdevpath, *nmactconnpath;
+	const gchar *nmdevpath/*, *nmactconnpath*/;
 	guint nmdevtype;
 	
 	if ((mmguicore == NULL) || (device == NULL)) return FALSE;
@@ -259,6 +258,10 @@ G_MODULE_EXPORT gboolean mmgui_module_device_connection_open(gpointer mmguicore,
 	//First time just set it to NULL
 	moduledata->nmdevpath = NULL;
 	
+	/*Initialize local variables*/
+	nmdevtype = MODULE_INT_DEVICE_TYPE_UNKNOWN;
+	nmdevpath = NULL;
+	
 	//Network Manager interface
 	if (moduledata->nmproxy != NULL) {
 		nmdevices = g_dbus_proxy_call_sync(moduledata->nmproxy, "GetDevices", NULL, 0, -1, NULL, &error);
@@ -295,12 +298,12 @@ G_MODULE_EXPORT gboolean mmgui_module_device_connection_open(gpointer mmguicore,
 							g_variant_unref(devproperties);
 						}
 						/*Active connection*/
-						devproperties = g_dbus_proxy_get_cached_property(nmdevproxy, "ActiveConnection");
+						/*devproperties = g_dbus_proxy_get_cached_property(nmdevproxy, "ActiveConnection");
 						if (devproperties != NULL) {
 							strlength = 256;
 							nmactconnpath = g_variant_get_string(devproperties, &strlength);
 							g_variant_unref(devproperties);
-						}
+						}*/
 						/*Is it device we looking for*/
 						if ((nmdevtype == MODULE_INT_DEVICE_TYPE_MODEM) && (g_str_equal(device->objectpath, nmdevpath))) {
 							/*Cache device path*/
@@ -312,7 +315,7 @@ G_MODULE_EXPORT gboolean mmgui_module_device_connection_open(gpointer mmguicore,
 						}
 					} else {
 						//Failed to create Network Manager device proxy
-						mmgui_module_device_connection_get_timestamp(mmguicorelc);
+						/*mmgui_module_device_connection_get_timestamp(mmguicorelc);*/
 						g_error_free(error);
 					}
 					g_variant_unref(nmdevnodel2);
