@@ -19,6 +19,7 @@
 
 #include <stdlib.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <string.h>
 #include <gdbm.h>
@@ -168,35 +169,18 @@ void mmgui_smsdb_message_free(mmgui_sms_message_t message)
 	g_free(message);
 }
 
-static gchar *mmgui_smsdb_message_escape_number(const gchar *number)
-{
-	size_t srclen, numlen;
-	gchar *bcdnumber, *escnumber;
-	
-	if (number == NULL) return NULL;
-	
-	srclen = strlen(number);
-	
-	if (srclen == 0) return NULL;
-	
-	bcdnumber = bcd_to_utf8_ascii_part(number, srclen, &numlen);
-	
-	if (bcdnumber != NULL) {
-		escnumber = encoding_clear_special_symbols(bcdnumber, numlen);
-	} else {
-		escnumber = encoding_clear_special_symbols(g_strdup(number), srclen);
-	}
-	
-	return escnumber;
-}
-
 gboolean mmgui_smsdb_message_set_number(mmgui_sms_message_t message, const gchar *number)
 {
 	gchar *escnumber;
+	gsize srclen;
 	
 	if ((message == NULL) || (number == NULL)) return FALSE;
 	
-	escnumber = mmgui_smsdb_message_escape_number(number);
+	srclen = strlen(number);
+	
+	if (srclen == 0) return FALSE;
+	
+	escnumber = encoding_clear_special_symbols(g_strdup(number), srclen);
 	
 	if (escnumber != NULL) {
 		if (message->number != NULL) {
@@ -219,10 +203,15 @@ const gchar *mmgui_smsdb_message_get_number(mmgui_sms_message_t message)
 gboolean mmgui_smsdb_message_set_service_number(mmgui_sms_message_t message, const gchar *number)
 {
 	gchar *escnumber;
+	gsize srclen;
 	
 	if ((message == NULL) || (number == NULL)) return FALSE;
 	
-	escnumber = mmgui_smsdb_message_escape_number(number);
+	srclen = strlen(number);
+	
+	if (srclen == 0) return FALSE;
+	
+	escnumber = encoding_clear_special_symbols(g_strdup(number), srclen);
 	
 	if (escnumber != NULL) {
 		if (message->svcnumber != NULL) {
@@ -726,7 +715,7 @@ gboolean mmgui_smsdb_set_message_read_status(smsdb_t smsdb, gulong idvalue, gboo
 	gchar smsid[64];
 	gint idlen;
 	datum key, data;
-	gchar *node, *trailer, *newmsg;
+	gchar *node, /* *trailer,*/ *newmsg;
 	gchar newtrailer[64];
 	gint newtrailerlen;
 	gboolean res;
@@ -866,8 +855,7 @@ static void mmgui_smsdb_xml_get_value(GMarkupParseContext *context, const gchar
 {
 	mmgui_sms_message_t message;
 	gchar *numescstr, *textescstr;
-	gsize numlen;
-	
+		
 	message = (mmgui_sms_message_t)data;
 	
 	if (mmgui_smsdb_xml_parameter == MMGUI_SMSDB_XML_PARAM_NULL) return;
@@ -876,15 +864,9 @@ static void mmgui_smsdb_xml_get_value(GMarkupParseContext *context, const gchar
 		case MMGUI_SMSDB_XML_PARAM_NUMBER:
 			numescstr = encoding_unescape_xml_markup(text, size);
 			if (numescstr != NULL) {
-				message->number = bcd_to_utf8_ascii_part(numescstr, strlen(numescstr), &numlen);
-				g_free(numescstr);
+				message->number = encoding_clear_special_symbols((gchar *)numescstr, strlen(numescstr));
 			} else {
-				message->number = bcd_to_utf8_ascii_part(text, size, &numlen);
-			}
-			if (message->number != NULL) {
-				message->number = encoding_clear_special_symbols(message->number, numlen);
-			} else {
-				message->number = encoding_clear_special_symbols(g_strdup(text), size);
+				message->number = encoding_clear_special_symbols((gchar *)text, size);
 			}
 			break;
 		case MMGUI_SMSDB_XML_PARAM_TIME:
