@@ -19,6 +19,7 @@
 
 #include <stdlib.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <glib/gprintf.h>
 #include <string.h>
 #include <gdbm.h>
@@ -57,7 +58,7 @@ static void mmgui_trafficdb_xml_end_element(GMarkupParseContext *context, const
 
 #define	MMGUI_TRAFFICDB_LEAP_YEAR_FEBRUARY_DAYS   29
 
-static unsigned int mmgui_trafficdb_month_days[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
+static guint mmgui_trafficdb_month_days[12] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
 
 
 static guint mmgui_trafficdb_get_month_days(guint month, guint year)
@@ -84,7 +85,7 @@ static guint mmgui_trafficdb_get_year_days(guint year)
 	}
 }
 
-static time_t mmgui_trafficdb_get_new_day_timesatmp(time_t currenttime)
+time_t mmgui_trafficdb_get_new_day_timesatmp(time_t currenttime)
 {
 	struct tm *ltime;
 	time_t restime;
@@ -95,24 +96,34 @@ static time_t mmgui_trafficdb_get_new_day_timesatmp(time_t currenttime)
 	
 	/*Day of week*/
 	if ((ltime->tm_wday + 1) > MMGUI_TRAFFICDB_WEEK_END) {
+		/*Next week*/
 		ltime->tm_wday = MMGUI_TRAFFICDB_WEEK_BEGIN;
 	} else {
+		/*Next day*/
 		ltime->tm_wday++;
 	}
 	/*Year*/
 	if ((ltime->tm_yday + 1) >= mmgui_trafficdb_get_year_days(ltime->tm_year)) {
+		/*Next year*/
 		ltime->tm_year++;
 		ltime->tm_yday = 0;
 	} else {
+		/*Next day*/
 		ltime->tm_yday++;
 	}
 	/*Month*/
 	if ((ltime->tm_mday + 1) > mmgui_trafficdb_get_month_days(ltime->tm_mon, ltime->tm_year)) {
+		/*New month*/
 		if ((ltime->tm_mon + 1) > 11) {
+			/*New year*/
 			ltime->tm_mon = 0;
+		} else {
+			/*Next month*/
+			ltime->tm_mon++;
 		}
-		ltime->tm_mday = 0;
+		ltime->tm_mday = 1;
 	} else {
+		/*Next day in month*/
 		ltime->tm_mday++;
 	}
 	/*Hours, minutes, seconds*/
@@ -292,61 +303,81 @@ gboolean mmgui_trafficdb_traffic_update(mmgui_trafficdb_t trafficdb, mmgui_traff
 		
 	currenttime = time(NULL);
 	
-	if ((trafficdb->sessstate == MMGUI_TRAFFICDB_SESSION_STATE_NEW) && (!trafficdb->sessinitialized)) {
+	/*Initialization*/
+	if (!trafficdb->sessinitialized) {
 		/*Count time interval between day begin time and session start time*/
 		sessdeltatime = difftime(trafficdb->presdaytime, trafficdb->sesstime);
-		/*Set session initial values*/
-		trafficdb->sessrxbytes = update->fullrxbytes;
-		trafficdb->sesstxbytes = update->fulltxbytes;
-		trafficdb->sessduration = update->fulltime;
-		/*If session was started before day begin corrections needed*/
-		if (sessdeltatime > 0) {
-			/*Current session*/
-			trafficdb->sessduration -= sessdeltatime;
-			/*Past day session*/
-			pastdaytime = mmgui_trafficdb_truncate_day_timesatmp(trafficdb->sesstime);
-			pasttraffic = mmgui_trafficdb_day_traffic_read(trafficdb, pastdaytime);
-			if (pasttraffic != NULL) {
-				pasttraffic->sessduration += sessdeltatime;
-				/*Write to database*/
-				if (!mmgui_trafficdb_day_traffic_write(trafficdb, pasttraffic)) {
-					g_debug("Failed to write traffic statistics to database\n");
+		/*New session (first session of a day started day before or second session of a day)*/
+		if (trafficdb->sessstate == MMGUI_TRAFFICDB_SESSION_STATE_NEW) {
+			/*If session was started before day begin, corrections needed*/
+			if (sessdeltatime > 0.0) {
+				/*Past day session*/
+				pastdaytime = mmgui_trafficdb_truncate_day_timesatmp(trafficdb->sesstime);
+				pasttraffic = mmgui_trafficdb_day_traffic_read(trafficdb, pastdaytime);
+				if (pasttraffic != NULL) {
+					pasttraffic->sessrxbytes = update->fullrxbytes;
+					pasttraffic->sesstxbytes = update->fulltxbytes;
+					pasttraffic->sessduration = sessdeltatime;
+					/*Write to database*/
+					if (!mmgui_trafficdb_day_traffic_write(trafficdb, pasttraffic)) {
+						g_debug("Failed to write traffic statistics to database\n");
+					}
+					g_free(pasttraffic);
 				}
-				g_free(pasttraffic);
+				/*Current session*/
+				//trafficdb->sesstime = trafficdb->presdaytime;
+				trafficdb->sessrxbytes = 0;
+				trafficdb->sesstxbytes = 0;
+				trafficdb->sessduration = update->fulltime - sessdeltatime;
+			} else {
+				/*Set session initial values*/
+				trafficdb->sessrxbytes = update->fullrxbytes;
+				trafficdb->sesstxbytes = update->fulltxbytes;
+				trafficdb->sessduration = update->fulltime;
+			}
+		} else if (trafficdb->sessstate == MMGUI_TRAFFICDB_SESSION_STATE_OLD) {
+			/*Old session*/
+			/*If session was started before day begin, do not do anything*/
+			if (sessdeltatime <= 0.0) {
+				/*Update with actual information*/
+				trafficdb->sessrxbytes = update->fullrxbytes;
+				trafficdb->sesstxbytes = update->fulltxbytes;
+				trafficdb->sessduration = update->fulltime;
 			}
 		}
 		trafficdb->sessinitialized = TRUE;
-	}
-	
-	if (currenttime <= trafficdb->nextdaytime) {
+	} else {
+		/*Already initialized session*/
+		/*Write traffic statistics at the end of the day*/
+		if (currenttime > trafficdb->nextdaytime) {
+			/*Write current statistics*/
+			memset(&traffic, 0, sizeof(traffic));
+			traffic.daytime = trafficdb->presdaytime;
+			traffic.dayrxbytes = trafficdb->dayrxbytes;
+			traffic.daytxbytes = trafficdb->daytxbytes;
+			traffic.dayduration = trafficdb->dayduration;
+			traffic.sesstime = trafficdb->sesstime;
+			traffic.sessrxbytes = trafficdb->sessrxbytes;
+			traffic.sesstxbytes = trafficdb->sesstxbytes;
+			traffic.sessduration = trafficdb->sessduration;
+			/*Write to database*/
+			if (!mmgui_trafficdb_day_traffic_write(trafficdb, &traffic)) {
+				g_debug("Failed to write traffic statistics to database\n");
+			}
+			/*Correct values*/
+			trafficdb->dayrxbytes = 0;
+			trafficdb->daytxbytes = 0;
+			trafficdb->dayduration = 0;
+			trafficdb->sessrxbytes = update->deltarxbytes;
+			trafficdb->sesstxbytes = update->deltatxbytes;
+			trafficdb->sessduration = update->deltaduration;
+			trafficdb->presdaytime = mmgui_trafficdb_truncate_day_timesatmp(currenttime);
+			trafficdb->nextdaytime = mmgui_trafficdb_get_new_day_timesatmp(currenttime);
+		}
 		/*Count current values*/
 		trafficdb->sessrxbytes += update->deltarxbytes;
 		trafficdb->sesstxbytes += update->deltatxbytes;
 		trafficdb->sessduration += update->deltaduration;
-	} else {
-		/*Write current statistics*/
-		memset(&traffic, 0, sizeof(traffic));
-		traffic.daytime = trafficdb->presdaytime;
-		traffic.dayrxbytes = trafficdb->dayrxbytes;
-		traffic.daytxbytes = trafficdb->daytxbytes;
-		traffic.dayduration = trafficdb->dayduration;
-		traffic.sesstime = trafficdb->sesstime;
-		traffic.sessrxbytes = trafficdb->sessrxbytes;
-		traffic.sesstxbytes = trafficdb->sesstxbytes;
-		traffic.sessduration = trafficdb->sessduration;
-		/*Write to database*/
-		if (!mmgui_trafficdb_day_traffic_write(trafficdb, &traffic)) {
-			g_debug("Failed to write traffic statistics to database\n");
-		}
-		/*Correct values*/
-		trafficdb->dayrxbytes = 0;
-		trafficdb->daytxbytes = 0;
-		trafficdb->dayduration = 0;
-		trafficdb->sessrxbytes = update->deltarxbytes;
-		trafficdb->sesstxbytes = update->deltatxbytes;
-		trafficdb->sessduration = update->deltaduration;
-		trafficdb->presdaytime = mmgui_trafficdb_truncate_day_timesatmp(currenttime);
-		trafficdb->nextdaytime = mmgui_trafficdb_get_new_day_timesatmp(currenttime);
 	}
 	
 	return TRUE;
@@ -482,7 +513,6 @@ gboolean mmgui_trafficdb_day_traffic_write(mmgui_trafficdb_t trafficdb, mmgui_da
 {
 	GDBM_FILE db;
 	gchar dayid[64];
-	gulong idvalue;
 	gint idlen;
 	datum key, data;
 	gchar *daytrafficxml;
@@ -552,6 +582,8 @@ GSList *mmgui_trafficdb_get_traffic_list_for_month(mmgui_trafficdb_t trafficdb,
 	mmgui_day_traffic_t daytraffic;
 	datum key, data;
 	gchar dayid[64];
+	struct tm *timespec;
+	gboolean currentstatscorrected;
 	
 	if (trafficdb == NULL) return NULL;
 	if (trafficdb->filepath == NULL) return NULL;
@@ -559,31 +591,62 @@ GSList *mmgui_trafficdb_get_traffic_list_for_month(mmgui_trafficdb_t trafficdb,
 	begtime = mmgui_trafficdb_get_month_begin_timestamp(month, year);
 	endtime = mmgui_trafficdb_get_month_end_timestamp(month, year);
 	
-	db = gdbm_open((gchar *)trafficdb->filepath, 0, GDBM_READER, 0755, 0);
-	
-	if (db == NULL) return NULL;
-	
 	list = NULL;
 	
-	key = gdbm_firstkey(db);
+	currentstatscorrected = FALSE;
 	
-	if (key.dptr != NULL) {
-		do {
-			memset(dayid, 0, sizeof(dayid));
-			strncpy(dayid, key.dptr, key.dsize);
-			daytime = (time_t)strtoull(dayid, NULL, 10);
-			if ((difftime(endtime, daytime) >= 0.0) && (difftime(daytime, begtime) > 0.0)) {
-				data = gdbm_fetch(db, key);
-				if (data.dptr != NULL) {
-					daytraffic = mmgui_trafficdb_xml_parse(data.dptr, data.dsize);
-					list = g_slist_prepend(list, daytraffic);
+	db = gdbm_open((gchar *)trafficdb->filepath, 0, GDBM_READER, 0755, 0);
+	
+	if (db != NULL) {
+		/*Get data from database if possible*/
+		key = gdbm_firstkey(db);
+		
+		if (key.dptr != NULL) {
+			do {
+				memset(dayid, 0, sizeof(dayid));
+				strncpy(dayid, key.dptr, key.dsize);
+				daytime = (time_t)strtoull(dayid, NULL, 10);
+				if ((difftime(endtime, daytime) >= 0.0) && (difftime(daytime, begtime) > 0.0)) {
+					data = gdbm_fetch(db, key);
+					if (data.dptr != NULL) {
+						daytraffic = mmgui_trafficdb_xml_parse(data.dptr, data.dsize);
+						if ((daytime == trafficdb->presdaytime) && (trafficdb->sessactive)) {
+							/*Today statistics correction*/
+							daytraffic->dayrxbytes = trafficdb->dayrxbytes;
+							daytraffic->daytxbytes = trafficdb->daytxbytes;
+							daytraffic->dayduration = trafficdb->dayduration;
+							daytraffic->sessrxbytes = trafficdb->sessrxbytes;
+							daytraffic->sesstxbytes = trafficdb->sesstxbytes;
+							daytraffic->sessduration = trafficdb->sessduration;
+							currentstatscorrected = TRUE;
+						}
+						list = g_slist_prepend(list, daytraffic);
+					}
 				}
-			}
-			key = gdbm_nextkey(db, key);
-		} while (key.dptr != NULL);
+				key = gdbm_nextkey(db, key);
+			} while (key.dptr != NULL);
+		}
+		gdbm_close(db);
 	}
 	
-	gdbm_close(db);
+	/*If statistics for current day must be included*/
+	daytime = time(NULL);
+	timespec = localtime((const time_t *)&daytime);
+	
+	if ((month == timespec->tm_mon) && (!currentstatscorrected)) {
+		if ((trafficdb->dayduration + trafficdb->sessduration) > 0) {
+			daytraffic = g_new(struct _mmgui_day_traffic, 1);
+			daytraffic->daytime = trafficdb->presdaytime;
+			daytraffic->dayrxbytes = trafficdb->dayrxbytes;
+			daytraffic->daytxbytes = trafficdb->daytxbytes;
+			daytraffic->dayduration = trafficdb->dayduration;
+			daytraffic->sesstime = trafficdb->sesstime;
+			daytraffic->sessrxbytes = trafficdb->sessrxbytes;
+			daytraffic->sesstxbytes = trafficdb->sesstxbytes;
+			daytraffic->sessduration = trafficdb->sessduration;
+			list = g_slist_prepend(list, daytraffic);
+		}
+	}
 	
 	if (list != NULL) {
 		list = g_slist_sort(list, mmgui_trafficdb_traffic_list_sort_compare);
